@@ -4,7 +4,7 @@ use 5.008000;
 use strict;
 use warnings;
 
-our $VERSION = '0.19_01';
+our $VERSION = '0.21_01';
 
 use File::Spec;
 use YAML::XS qw( LoadFile );
@@ -94,7 +94,7 @@ sub load {
     $self->{_config} = $self->{_merger}->merge( $self->{_config},
         { ENV => {%ENV} } );
   }
-  $self->_process_tree( $self->{_config} );
+  $self->_process_tree( $self->{_config}, [] );
 
   $self->{_vars}       = {};
   $self->{_seen_nodes} = {};
@@ -194,10 +194,11 @@ sub _load_json {
 
 sub _process_tree {
   my $self = shift;
+  my $ancs = pop;
 
   return if readonly( $_[0] );
 
-  $_[0] = $self->_process_node( $_[0] );
+  $_[0] = $self->_process_node( $_[0], $ancs );
 
   if ( my $node_addr = refaddr( $_[0] ) ) {
     return if $self->{_seen_nodes}{$node_addr};
@@ -207,12 +208,12 @@ sub _process_tree {
 
   if ( ref( $_[0] ) eq 'HASH' ) {
     foreach ( values %{ $_[0] } ) {
-      $self->_process_tree($_);
+      $self->_process_tree( $_, [ $_[0], @{$ancs} ] );
     }
   }
   elsif ( ref( $_[0] ) eq 'ARRAY' ) {
     foreach ( @{ $_[0] } ) {
-      $self->_process_tree($_);
+      $self->_process_tree( $_, [ $_[0], @{$ancs} ] );
     }
   }
 
@@ -222,16 +223,17 @@ sub _process_tree {
 sub _process_node {
   my $self = shift;
   my $node = shift;
+  my $ancs = shift;
 
   return unless defined $node;
 
   if ( !ref($node) && $self->{interpolate_variables} ) {
     $node =~ s/\$((\$?)\{([^\}]*)\})/
-        $2 ? $1 : ( $self->_resolve_var( $3 ) || '' )/ge;
+        $2 ? $1 : ( $self->_resolve_var( $3, $ancs ) || '' )/ge;
   }
   elsif ( ref($node) eq 'HASH' && $self->{process_directives} ) {
     if ( defined $node->{var} ) {
-      $node = $self->_resolve_var( $node->{var} );
+      $node = $self->_resolve_var( $node->{var}, $ancs );
     }
     elsif ( defined $node->{include} ) {
       $node = $self->_build_tree( $node->{include} );
@@ -239,13 +241,13 @@ sub _process_node {
     else {
       if ( defined $node->{underlay} ) {
         my $layer = delete $node->{underlay};
-        $layer = $self->_process_layer($layer);
+        $layer = $self->_process_layer( $layer, $ancs );
         $node = $self->{_merger}->merge( $layer, $node );
       }
 
       if ( defined $node->{overlay} ) {
         my $layer = delete $node->{overlay};
-        $layer = $self->_process_layer($layer);
+        $layer = $self->_process_layer( $layer, $ancs );
         $node = $self->{_merger}->merge( $node, $layer );
       }
     }
@@ -257,15 +259,16 @@ sub _process_node {
 sub _process_layer {
   my $self  = shift;
   my $layer = shift;
+  my $ancs  = shift;
 
   if ( ref($layer) eq 'HASH' ) {
-    $layer = $self->_process_node( $layer );
+    $layer = $self->_process_node( $layer, $ancs );
   }
   elsif ( ref($layer) eq 'ARRAY' ) {
     my $new_layer = {};
 
     foreach my $node ( @{$layer} ) {
-      $node = $self->_process_node( $node );
+      $node = $self->_process_node( $node, $ancs );
       $new_layer = $self->{_merger}->merge( $new_layer, $node );
     }
 
@@ -276,63 +279,113 @@ sub _process_layer {
 }
 
 sub _resolve_var {
-  my $self     = shift;
-  my $var_name = shift;
+  my $self = shift;
+  my $name = shift;
+  my $ancs = shift;
 
-  my $vars = $self->{_vars};
-
-  unless ( defined $vars->{$var_name} ) {
-    my @tokens = split( /\./, $var_name );
-    my $pointer = $self->{_config};
-
-    my $value;
+  if ( $name =~ m/^\./ ) {
+    my @tokens    = split( /\./, $name );
+    my $anc_index = -1;
 
     while (1) {
-      my $token = shift @tokens;
+      my $token = $tokens[0];
       $token =~ s/^\s+//;
       $token =~ s/\s+$//;
 
-      if ( ref($pointer) eq 'HASH' ) {
-        last unless defined $pointer->{$token};
+      last if length($token) > 0;
 
-        if ( !@tokens ) {
-          $value = $self->_process_node( $pointer->{$token} );
-          $pointer->{$token} = $value;
-
-          last;
-        }
-
-        last unless ref( $pointer->{$token} );
-
-        $pointer = $pointer->{$token};
-      }
-      else { # ARRAY
-        if ( $token =~ m/\D/ ) {
-          die "Argument \"$token\" isn't numeric in array element:"
-              . " $var_name\n";
-        }
-
-        last unless defined $pointer->[$token];
-
-        if ( !@tokens ) {
-          $value = $self->_process_node( $pointer->[$token] );
-          $pointer->[$token] = $value;
-
-          last;
-        }
-
-        last unless ref( $pointer->[$token] );
-
-        $pointer = $pointer->[$token];
-      }
+      shift @tokens;
+      $anc_index++;
     }
 
-    if ( defined $value ) {
-      $vars->{$var_name} = $value;
+    my $node = $ancs->[$anc_index];
+
+    my $value = eval {
+      $self->_fetch_value( $node, \@tokens, $ancs );
+    };
+
+    if ($@) {
+      chomp $@;
+      die qq{Can't resolve variable "$name"; $@\n};
+    }
+
+    return $value;
+  }
+
+  my $vars = $self->{_vars};
+
+  unless ( defined $vars->{$name} ) {
+    my @tokens = split( /\./, $name );
+
+    my $value = eval {
+      $self->_fetch_value( $self->{_config}, \@tokens, $ancs );
+    };
+
+    if ($@) {
+      chomp $@;
+      die qq{Can't resolve variable "$name"; $@\n};
+    }
+
+    $vars->{$name} = $value;
+  }
+
+  return $vars->{$name};
+}
+
+####
+sub _fetch_value {
+  my $self   = shift;
+  my $node   = shift;
+  my $tokens = shift;
+  my $ancs   = shift;
+
+  my $value;
+  my @anc_stack = @{$ancs};
+
+  while (1) {
+    my $token = shift @{$tokens};
+    $token =~ s/^\s+//;
+    $token =~ s/\s+$//;
+
+    if ( ref($node) eq 'HASH' ) {
+      last unless defined $node->{$token};
+
+      unshift( @anc_stack, $node );
+
+      unless ( @{$tokens} ) {
+        $value = $self->_process_node( $node->{$token}, \@anc_stack );
+        $node->{$token} = $value;
+
+        last;
+      }
+
+      last unless ref( $node->{$token} );
+
+      $node = $node->{$token};
+    }
+    else { # ARRAY
+      if ( $token =~ m/\D/ ) {
+        die qq{Argument "$token" isn't numeric in array element.\n};
+      }
+
+      last unless defined $node->[$token];
+
+      unshift( @anc_stack, $node );
+
+      unless ( @{$tokens} ) {
+        $value = $self->_process_node( $node->[$token], \@anc_stack );
+        $node->[$token] = $value;
+
+        last;
+      }
+
+      last unless ref( $node->[$token] );
+
+      $node = $node->[$token];
     }
   }
 
-  return $vars->{$var_name};
+  return $value;
 }
 
 1;
@@ -501,20 +554,22 @@ After merging of two files we will get:
 =head1 INTERPOLATION
 
 Config::Processor can interpolate variables in string values (if you need alias
-for complex structures see C<var> directive). For example, we have
-F<myapp.yml> file:
+for complex structures see C<var> directive). Variable names can be absolute or
+relative. Relative variable names begins with dot "." The number of dots
+depends on the nesting level of the current configuration parameter relative to
+referenced configuration parameter.
 
   myapp:
     media_formats: [ "images", "audio", "video" ]
 
     dirs:
       root_dir: "/myapp"
-      templates_dir: "${myapp.dirs.root_dir}/templates"
-      sessions_dir: "${myapp.dirs.root_dir}/sessions"
+      templates_dir: "${.root_dir}/templates"
+      sessions_dir: "${.root_dir}/sessions"
       media_dirs:
-        - "${myapp.dirs.root_dir}/media/${myapp.media_formats.0}"
-        - "${myapp.dirs.root_dir}/media/${myapp.media_formats.1}"
-        - "${myapp.dirs.root_dir}/media/${myapp.media_formats.2}"
+        - "${..root_dir}/media/${myapp.media_formats.0}"
+        - "${..root_dir}/media/${myapp.media_formats.1}"
+        - "${..root_dir}/media/${myapp.media_formats.2}"
 
 After processing of the file we will get:
 
@@ -548,10 +603,14 @@ After processing we will get:
 =item var: varname
 
 Assigns configuration parameter value to another configuration parameter.
+Variable names in the directive can be absolute or relative. Relative variable
+names begins with dot ".". The number of dots depends on the nesting level of
+the current configuration parameter relative to referenced configuration
+parameter.
 
   myapp:
     db:
-      generic_options:
+      default_options:
         PrintWarn:  0
         PrintError: 0
         RaiseError: 1
@@ -563,7 +622,7 @@ Assigns configuration parameter value to another configuration parameter.
           dbname:   "stat"
           username: "stat_writer"
           password: "stat_writer_pass"
-          options: { var: myapp.db.generic_options }
+          options: { var: myapp.db.default_options }
 
         stat_slave:
           host:     "stat-slave.mydb.com"
@@ -571,7 +630,7 @@ Assigns configuration parameter value to another configuration parameter.
           dbname:   "stat"
           username: "stat_reader"
           password: "stat_reader_pass"
-          options: { var: myapp.db.generic_options }
+          options: { var: ...default_options }
 
 =item include: filename
 
@@ -612,13 +671,13 @@ For example, you can use this directive to set default values of parameters.
             RaiseError: 1
 
         stat_master:
-          underlay: { var: myapp.db.connectors.default }
+          underlay: { var: .default }
           host:     "stat-master.mydb.com"
           username: "stat_writer"
           password: "stat_writer_pass"
 
         stat_slave:
-          underlay: { var: myapp.db.connectors.default }
+          underlay: { var: .default }
           host:     "stat-slave.mydb.com"
           username: "stat_reader"
           password: "stat_reader_pass"
@@ -633,19 +692,19 @@ You can move default parameters in separate files.
           - { include: db_connectors/default_test.yml }
 
         stat_master:
-          underlay: { var: myapp.db.connectors.default }
+          underlay: { var: .default }
           host:     "stat-master.mydb.com"
           username: "stat_writer"
           password: "stat_writer_pass"
 
         stat_slave:
-          underlay: { var: myapp.db.connectors.default }
+          underlay: { var: .default }
           host:     "stat-slave.mydb.com"
           username: "stat_reader"
           password: "stat_reader_pass"
 
         test:
-          underlay: { var: myapp.db.connectors.default_test }
+          underlay: { var: .default_test }
           username: "test"
           password: "test_pass"
 
@@ -675,18 +734,18 @@ configuration parameters.
           port: "4321"
 
         stat_master:
-          underlay: { var: myapp.db.connectors.default }
+          underlay: { var: .default }
           host:     "stat-master.mydb.com"
           username: "stat_writer"
           password: "stat_writer_pass"
-          overlay:  { var: myapp.db.connectors.test }
+          overlay:  { var: .test }
 
         stat_slave:
-          underlay: { var: myapp.db.connectors.default }
+          underlay: { var: .default }
           host:     "stat-slave.mydb.com"
           username: "stat_reader"
           password: "stat_reader_pass"
-          overlay:  { var: myapp.db.connectors.test }
+          overlay:  { var: .test }
 
 To disable overriding just assign to C<test> connector empty hash.
 
